\name{pnmath}
\title{Parallelized Vector Math Functions}
\alias{pnmath0-package}
\alias{getNumProcs}
\alias{getNumPnmathThreads}
\alias{setNumPnmathThreads}
\alias{getParallelThresholds}
\alias{calibratePnmath}
\usage{
getNumProcs()
getNumPnmathThreads()
setNumPnmathThreads(n)
getParallelThresholds()
calibratePnmath(m, N = 100000)
}
\arguments{
  \item{n}{integer maximal number of threads to use}
  \item{m}{optional positive real value specifying the parallel
    overhead; should probably be between 100 and 1000 for most systems.}
  \item{N}{sample size used in the calibration; should be at least
    100000 for reliable results.}
}
\description{
  Parallelized vector math functions and support routines.
}
\details{
  Loading this package replaces a number of vector math functions with
  versions that are parallelized using pthreads.  The functions will be
  run using multiple threads if their results will be long enough for
  the parallel overhead to be outweighed by the parallel gains.  The
  function \code{getParallelThresholds} returns the cutoffs used.  On
  load a calibration calculation is carried out to asses the parallel
  overhead and adjust these thresholds.

  The function \code{getNumProcs} returns the number of processors if
  this can be determined, or \code{NA} if it cannot.

  The function \code{getNumPnmathThreads} returns the maximal number of
  threads that will be used.  By default this is the number of
  processors or the value of the environment variable
  \code{OMP_NUM_THREADS}, whichever is smaller.  The function
  \code{setNumPnmathThreads} can be used to set the maximal number of
  threads to use.

  \code{calibratePnmath} can be used to manually set the overhead value
  to use to run a calibration experiment to determine the appropriate
  value.  Thresholds are adjusted and the supplied or computed overhead
  value is returned.  This value represents the additional time needed
  to run two threads with no work in units of a single \code{dnorm}
  calculation.  For (at least some) Linux machines 200 seems a good
  value, and for OS X 500 seems appropriate.  This means on these
  machines the time taken to manage the second thread is 200 or 500
  times the time taken to compute \code{dnorm} on a scalar argument.
}
\keyword{internal}
